"""Module for unit conversions inside of Blueprints."""

# <editor-fold desc="Forces conversion">
KN_TO_N = 1e3
N_TO_KN = 1e-3
MM3_TO_M3 = 1e-9
MPA_TO_KPA = 1e3
# </editor-fold>

# <editor-fold desc="Section modulus conversion">
CM3_TO_MM3 = 1e3
# </editor-fold>

# <editor-fold desc="Moments conversion">
NMM_TO_KNM = 1e-6
# </editor-fold>
