"""Module to keep track of type aliases used in Blueprints."""

# <editor-fold desc="DISTANCES">
MM = float
CM = float
M = float
KM = float
# </editor-fold>

# <editor-fold desc="ROTATIONS">
DEG = float
RAD = float
# </editor-fold>

# <editor-fold desc="WEIGHTS">
KG = float
G = float
# </editor-fold>

# <editor-fold desc="FORCES">
N = float
KN = float
KN_M = float
# </editor-fold>

# <editor-fold desc="MOMENTS">
NM = float
KNM = float
# </editor-fold>

# <editor-fold desc="STRESSES">
MPA = float
KPA = float
GPA = float
# </editor-fold>

# <editor-fold desc="AREAS">
MM2 = float
CM2 = float
DM2 = float
M2 = float
KM2 = float
# </editor-fold>

# <editor-fold desc="VOLUMES">
MM3 = float
CM3 = float
M3 = float
# </editor-fold>

# <editor-fold desc="BI-QUADRATIC">
MM4 = float
CM4 = float
M4 = float
# </editor-fold>

# <editor-fold desc="TIME">
HOURS = float
MINUTES = float
SECONDS = float
DAYS = float
WEEKS = float
MONTHS = float
YEARS = float
# </editor-fold>

# <editor-fold desc="Ratio's">
RATIO = float
PERCENTAGE = float
DIMENSIONLESS = float
# </editor-fold>
